<?php

/**
 * Implementation of hook_content_edit_form().
 */
function dependent_fields_content_edit_form(&$form, &$form_state) {
  // Only support modules that implement hook_dependent_fields_widgets().
  $field = $form['#field'];
  $widget = $field['widget'];

  $widget_type = isset($widget['widget_type']) ? $widget['widget_type'] : $widget['type'];
  if (!in_array($widget_type, dependent_fields_allowed_widgets($field['type'], 'controlled'))) {
    return;
  }

  $type = (array)content_types($form['type_name']['#value']);

  // Set the weights for the form elements, so we can get the dependent fields settings
  // in the right place.
  
  $form['basic']['#weight'] = 0;
  $form['widget']['#weight'] = 1;
  $form['field']['#weight'] = 3;
  $form['submit']['#weight'] = 4;
  
  $form['#after_build'][] = 'dependent_fields_content_edit_form_after_build_include';
  $form['#validate'][] = 'dependent_fields_content_edit_form_validate';

  dependent_fields_settings_form($form, $form_state, $widget, $field, $type);

  return;
}

/**
 * Implementation of hook_widget_settings_alter().
 */

function _dependent_fields_widget_settings_alter(&$settings, $op, $widget) {
  // Only support modules that implement hook_dependent_fields_widgets().
  $widget_type = isset($widget['widget_type']) ? $widget['widget_type'] : $widget['type'];
  $allowed_widgets = dependent_fields_allowed_widgets('', 'controlled');

  if (!in_array($widget_type, $allowed_widgets)) {
    return;
  }

  // We don't use the form op here as it doesn't supply type and field information.
  // Instead we use the content_edit_form to create the dependent_fields_widget_
  // settings_form hook.

  if ($op == 'save') {
    $settings[] = 'dependent_fields';
  }
}

/**
 * Configuration form for editing the settings for a Dependent Field.
 */
function dependent_fields_settings_form(&$form, &$form_state, $widget, $field, $type) {
  $form['#cache'] = TRUE;
  $controller_field = isset($widget['dependent_fields']['controller_field']) ? $widget['dependent_fields']['controller_field'] : '';
  $hide_if_empty = isset($widget['dependent_fields']['hide_if_empty']) ? $widget['dependent_fields']['hide_if_empty'] : FALSE;
  $hide_none = isset($widget['dependent_fields']['hide_none']) ? $widget['dependent_fields']['hide_none'] : TRUE;

  $controlled_fields = dependent_fields_controlled_fields($field, $type);

  // Add the fieldset to the form
  $form['dependent_fields'] = array(
    '#type' => 'fieldset',
    '#title' => t('Dependent Field Settings'),
    '#collapsible' => TRUE,
    '#collapsed' => empty($controller_field) && empty($controlled_fields),
    '#weight' => 2,
    '#attributes' => array('id' => 'dependent-field-settings'),
    '#tree' => TRUE,
  );

  // Settings wrapper to contain all the dependent settings
  $form['dependent_fields']['settings'] = array(
    '#prefix' => '<div id="dependent-fields-settings-ahah">',
    '#suffix' => '</div>',
    '#weight' => 5,
  );

  $fields = dependent_fields_list(FALSE);

  $potential_fields = dependent_fields_potential_controller_options($field, $type);


  if (!empty($potential_fields)) {

    $form['dependent_fields']['controller_field'] = array(
      '#type' => 'select',
      '#title' => t('Controller Field'),
      '#description' => t('Choose a field to act as a controller field.'),
      '#options' => $potential_fields,
      '#default_value' => $controller_field,
      '#ahah' => array(
        'path' => 'dependent_fields/settings_callback',
        'wrapper' => 'dependent-fields-settings-ahah',
        'method' => 'replace',
        'effect' => 'fade',
      ),
      '#weight' => 0,
    );

    $form['dependent_fields']['change'] = array(
      '#type' => 'submit',
      '#value' => t('Change controller field'),
      '#attributes' => array('class' => 'ahah-fallback-button'),
      '#submit' => array('dependent_fields_settings_form_submit_update_controller'),
      '#prefix' => '<div class="fallback-button-wrapper">',
      '#suffix' => '</div>',
      '#weight' => 1,
    );

    $form['dependent_fields']['settings']['hide_if_empty'] = array(
      '#type' => 'checkbox',
      '#title' => t('Hide if Empty'),
      '#description' => t('Should this field be hidden if the list of allowed options is empty.'),
      '#default_value' => $hide_if_empty,
    );
    $form['dependent_fields']['settings']['hide_none'] = array(
      '#type' => 'checkbox',
      '#title' => t('Hide None Option'),
      '#description' => t('Should the None option be hidden from select and button lists'),
      '#default_value' => $hide_none,
    );
    $params = array($widget, $field, $type);
    $form['dependent_fields']['settings'] = array_merge($form['dependent_fields']['settings'], dependent_fields_invoke_field($field['type'], 'widget_form', $params));
  }
  else {
    $form['dependent_fields']['controller_field'] = array(
      '#type' => 'value',
      '#default_value' => '',
    );
    $form['dependent_fields']['controller_field_empty'] = array(
      '#type' => 'markup',
      '#value' => t('No suitable controller fields in this content type.'),
      '#prefix' => '<p class="dependent-field-empty-text">',
      '#suffix' => '</p>',
    );
  }

  $controlled_fields = dependent_fields_controlled_fields_list($field, $type);

  if (!empty($controlled_fields)) {
    $form['dependent_fields']['controlled_fields'] = array(
      '#type' => 'markup',
      '#value' => t('This field acts as a controller field to the following fields:') . theme_item_list($controlled_fields),
      '#prefix' => '<div class="dependent-fields-controlled-fields">',
      '#suffix' => '</div>',
    );
  }

  
  // Add a callback handler to other fields on the form, such as required.
  if ($form['field']['required']['#type'] == 'checkbox') {
    $form['field']['required']['#ahah'] = array(
      'path' => 'dependent_fields/settings_callback',
      'wrapper' => 'dependent-fields-settings-ahah',
      'method' => 'replace',
      'effect' => 'fade',
    );
  }

}


/**
 * Button submit handler.
 */
function dependent_fields_settings_form_submit_update_controller(&$form, &$form_state) {
  $form_state['change_controller'] = TRUE;
  $form_state['storage'] = $form_state['values'];
  $form_state['rebuild'] = TRUE;
}


/**
 * Validate a field's settings.
 */
function dependent_fields_content_edit_form_validate(&$form, &$form_state) {
  $form_values = $form_state['values'];
  if (isset($form_state['change_controller']) || $form_values['op'] == t('Change controller field')) {
    return;
  }
}

function dependent_fields_content_edit_form_after_build($form, &$form_state) {
  $field = $form['#field'];
  $type = (array)content_types($form['type_name']['#value']);
  $controlled_fields = dependent_fields_controlled_fields($field, $type);

  // Check to see if this has been submitted using the fallback button.
  if ($form_state['change_controller']) {
    // It has so we need to use the form_state['storage'] instead of ['values']
    $form_state['values'] = $form_state['storage'];
  }
  $values = $form_state['values']['dependent_fields'];  
  
  $controller_field = $values['controller_field'];

  // Update the value of the controller field element.
  $form['dependent_fields']['controller_field']['#value'] = $controller_field;

  // Add the css to hide the fallback button.
  dependent_fields_add();

  // Hide settings if no field is selected.
  if (empty($controller_field)) {
    // Replace the settings wrapper with a hidden field as we still need the wrapper div.
    $form['dependent_fields']['settings'] = array(
      '#prefix' => '<div id="dependent-fields-settings-ahah">',
      '#suffix' => '</div>',
      '#weight' => 5,
      '#type'   => 'hidden',
      '#value'  =>  NULL,
    );
  }
  else {
    $settings = &$form['dependent_fields']['settings'];
    // If the field is set to required, then hide the hide_none & hide_empty option.
    if ($form_state['values']['required']) {
      if (!empty($settings['hide_none'])){
        $settings['hide_none']['#type'] = 'hidden';
        $settings['hide_none']['#value'] = FALSE;
      }
      if (!empty($settings['hide_if_empty'])){
        $settings['hide_if_empty']['#type'] = 'hidden';
        $settings['hide_if_empty']['#value'] = FALSE;
      }
    }
  }

  // Make sure our submit handler runs first.
  array_unshift($form['#submit'], 'dependent_fields_content_edit_form_submit');

  return $form;
}

/**
 * Handle saving of field settings.
 */
function dependent_fields_content_edit_form_submit($form, &$form_state) {
  // Move settings from the settings key into the root.
  $form_values = &$form_state['values']['dependent_fields'];
  if (!empty($form_values) &&  is_array($form_values['settings'])) {
    foreach ($form_values['settings'] as $key => $value) {
      $form_values[$key] = $value;
    }
       
    unset($form_values['settings']);
  }

}
