<?php

/**
 * A function to check whether the current element is managed by dependendent fields.
 */
function dependent_fields_get_element_dependent_field($element, $type = 'both') {
  // Check to see if this element is registered to one of our fields.
  if (!empty($element['#field_name']) && !empty($element['#type_name'])) {
    $field = content_fields($element['#field_name'], $element['#type_name']);
    if (!empty($field)) {
      // Make sure this element is an allowed widget
      $allowed_widgets = dependent_fields_allowed_widgets($field['type'], $type);
      if (in_array($field['widget']['type'], $allowed_widgets)) {
        return $field;
      }
    }
  }

  return FALSE;
}

 /**
 * A #process callback that we make sure is the first to run.
 */
function dependent_fields_element_preprocess($element, $edit, &$form_state, $form) {
  if (!empty($element['#field_name'])) {
    $handler = $form_state['dependent_handler'];
    $info = $handler->field_info[$element['#field_name']];
    if (!empty($info)) {
      $field = $info['field'];
      $params = array($element, $info, $form_state, $form);
      $element = array_merge($element, dependent_fields_invoke_field($field['type'], 'element_preprocess', $params));
    }
  }  
  return $element;
}


/**
 * An #after_build function for an controlled field element
 *
 */
function dependent_fields_element_after_build($element, $form_state) {
  if (!empty($element['#field_name'])) {
    $handler = $form_state['dependent_handler'];
    $info = $handler->field_info[$element['#field_name']];
    if (!empty($info)) {
      $field = $info['field'];
      $params = array($element, $info, $form_state, $form);
      $element = array_merge($element, dependent_fields_invoke_field($field['type'], 'element_after_build', $params));
    }
  }
  return $element;
}

 /**
 * A #process callback to extend the dependent field element type.
 */
function dependent_fields_element_process($element, $edit, &$form_state, $form) {
  if (!empty($element['#field_name'])) {
    $handler = $form_state['dependent_handler'];
    $info = $handler->field_info[$element['#field_name']];
    if (!empty($info)) {
      $field = $info['field'];
      $params = array($element, $info, $form_state, $form);
      $element = array_merge($element, dependent_fields_invoke_field($field['type'], 'element_process', $params));
    }
  }
  return $element;
}


 /**
 * An #element_validate callback to extend the dependent field element type.
 */
function dependent_fields_element_validate($element, &$form_state, $form) {
  if (!empty($element['#field_name'])) {
    $handler = $form_state['dependent_handler'];
    $info = $handler->field_info[$element['#field_name']];
    if (!empty($info)) {
      $field = $info['field'];
      $params = array($element, $info, $form_state, $form);
      $element = array_merge($element, dependent_fields_invoke_field($field['type'], 'element_validate', $params));
    }
  }
  return $element;
}

/**
 * A #process callback to extend the optionwidgets type.
 */
function dependent_fields_optionwidgets_process($element, $edit, &$form_state, $form) {
  // Don't retrieve the allowed values if we are processing a copy of the form.
  if ($form_state['#dependent_copy']) {
    return $element;
  }
  // Make sure we have the form state set
  $handler = $form_state['dependent_handler'];

  if (empty($handler)) {
    return $element;
  }

  $info = $handler->field_info[$element['#field_name']];

  if (!empty($info)) {
    $column = $element['#columns'][0];
    $options = &$element[$column]['#options'];
    $option_count = count($element[$column]['#options']);
    $value = &$element['#value'][$column];
    $value_empty = TRUE;
    $value_match = FALSE;
    // we are dealing with multiple values.
    if (is_array($value)) {
      foreach ($value as $k => $v) {
        if (isset($v)) {
          $value_empty = FALSE;
          if (isset($options[$v])) {
            $value_match = TRUE;
          }
        }
      }
    }
    else {
      $value_empty = empty($value);
      $value_match = isset($options[$value]);
    }
    
    if ($info['hide_if_empty']) {
      if ($option_count <= 0 || ($option_count == 1 && !empty($options[''])) ) {
        $element['#type'] = 'hidden';
        unset($element['#value']);
        form_set_value($element, $element['#value'], $form_state);
        //dependent_fields_set_form_state($form_state);
        return $element;
      }
    }
    if ($option_count > 0 && !$value_empty) {
      if (!$value_match) {
        $value = NULL;
        form_set_value($element, $value, $form_state);
      }
    }
    if ($info['hide_none']) {
      unset($element[$column]['#options']['']);
      if ($option_count > 0 && empty($value)) {
        reset($options);
        $element['#value'][$column] = key($options);
        form_set_value($element, $element['#value'][$column], $form_state);
      }
    }
  
    //dependent_fields_set_form_state($form_state);
  }


  return $element;
}

/**
 * An #after_build function for a controller element
 *
 */
function dependent_fields_controller_after_build($element, $form_state) {
  // Check to see if this element is registered to one of our fields.
  if (!empty($element['#controller_field_name'])) {
    $handler = $form_state['dependent_handler'];
    $info = $handler->field_info[$element['#controller_field_name']];
    if (!empty($info['controlled_fields'])) {
      $field = $info['field'];
      dependent_fields_set_controller_elements($element, $field, 'controller');
    }
  }
  return $element;
}


/**
 * Locate the actual field element, and apply the controller ahah settings
 *
 */
function dependent_fields_set_controller_elements(&$form, &$field, $type = 'controller') {
  $field_columns = array_keys($field['columns']);
  $field_column = $field_columns[0];
  // Get a list of elements that we can apply ahah settings on.
  $valid_field_elements = dependent_fields_get_ahah_elements($field);

  foreach (element_children($form) as $key) {
    dependent_fields_set_controller_elements($form[$key], $field, $type);
  }
  // Check to see if we have a valid form element here
  if (in_array($form['#type'], $valid_field_elements)) {
    dependent_fields_set_ahah_settings($form, $field);
    $form = form_expand_ahah($form);
  }
}


function dependent_fields_transpose_form_values($field, $field_key, $items) {
  $return = array();
  $widget_type = $field['widget']['type'];

  if (in_array($widget_type, array('optionwidgets_buttons','optionwidgets_select', 'content_taxonomy_options'))) {
    $options = optionwidgets_options($field);   
  }
  if (is_array($items[$field_key])) {
    foreach ($items[$field_key] as $delta => $value) {
      if (in_array($widget_type, array('optionwidgets_buttons', 'content_taxonomy_options'))) {
        if (!empty($value) && array_key_exists($delta, $options)) {
          $return[][$field_key] = $delta;
        }
      }
      elseif ($widget_type == 'optionwidgets_select') {
        if (array_key_exists($value, $options)) {
          $return[][$field_key] = $value;
        }
      }
      else {
        if (count($value) || (is_string($value) && strlen(trim($value)) > 0)) {
          $return[$delta][$field_key] = $delta;
        }
      }
    }
    return $return;
  }
  return $items;
}